<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\EventListener;

use Dedi\SyliusMarketingPlugin\ControllerEvent\SelectItemEvent;
use Dedi\SyliusMarketingPlugin\Extractor\RequestProductExtractorInterface;
use Dedi\SyliusMarketingPlugin\Factory\Events\NavigationEventFactory;
use Dedi\SyliusMarketingPlugin\Storage\EventsBag;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Core\Model\ProductInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\ControllerArgumentsEvent;
use Symfony\Contracts\EventDispatcher\Event;

class NavigationEventListener implements EventSubscriberInterface
{
    private NavigationEventFactory $navEventFactory;
    private EventsBag $eventsStorage;
    private RequestProductExtractorInterface $requestProductExtractor;
    private string $searchRouteName;

    public function __construct(
        NavigationEventFactory $navEventFactory,
        EventsBag $eventsStorage,
        RequestProductExtractorInterface $requestProductExtractor,
        string $searchRouteName
    ) {
        $this->navEventFactory = $navEventFactory;
        $this->eventsStorage = $eventsStorage;
        $this->requestProductExtractor = $requestProductExtractor;
        $this->searchRouteName = $searchRouteName;
    }
    public static function getSubscribedEvents(): array
    {
        return [
            'sylius.product.show' => 'viewItem',
            'sonata.block.event.sylius.shop.product.index.before_list' => 'viewItemList',
            'controller_select_item' => 'selectItem',
            'kernel.controller_arguments' => 'onKernelRequest',
        ];
    }

    public function onKernelRequest(ControllerArgumentsEvent $event): void
    {
        $routeName = $event->getRequest()->attributes->get('_route');

        if ($this->searchRouteName !== $routeName ||
            !$event->isMainRequest() ||
            !\is_array($event->getController())
        ) {
            return;
        }

        $searchTerm = $this->getSearchTerm($event->getRequest());
        if (null !== $searchTerm) {
            $this->eventsStorage->setEvent(
                $this->navEventFactory->search($searchTerm),
            );
        }
    }

    public function viewItem(ResourceControllerEvent $resourceControllerEvent): void
    {
        $product = $resourceControllerEvent->getSubject();

        if (!$product instanceof ProductInterface) {
            return;
        }

        $this->eventsStorage->setEvent(
            $this->navEventFactory->viewItem($product),
        );
    }

    public function viewItemList(Event $event): void
    {
        $products = $this->requestProductExtractor->extract($event);

        $this->eventsStorage->setEvent(
            $this->navEventFactory->viewItemList($products),
        );
    }

    public function selectItem(SelectItemEvent $selectItemEvent): void
    {
        $this->eventsStorage->setEvent(
            $this->navEventFactory->selectItem($selectItemEvent),
        );
    }

    private function getSearchTerm(Request $request): ?string
    {
        $criteria = $request->query->get('criteria');

        if (isset($criteria['search']['value'])) {
            return (string) $criteria['search']['value'];
        }

        return null;
    }
}
